/*
 * boss2.c
 *
 * Last modified: <2008/08/31 09:43:06 +0900> By SUMIDA
 */
#include <math.h>
#include <SDL.h>
#include "system.h"
#include "texture.h"
#include "ship.h"
#include "shot.h"
#include "enemy.h"
#include "effect.h"

#define SHIP_ACT_MASK          0x00FF
#define SHIP_ACT_MOVE          0x0001
#define SHIP_ACT_RUSH          0x0002
#define SHIP_ACT_BACK          0x0004
#define SHIP_ACT_CHARGE        0x0008
#define SHIP_ACT_DISCHARGE     0x0010

extern SHIP_MGR enemy_mgr;

/* NOTE : {X̍s

		SHIP_STAT_APPEAR = o

	SHIP_ACT_MOVE
		SHIP_STAT_MOVE   = ㉺ړA[U[Vbg
		SHIP_STAT_NORMAL = ҋ@
			SHIP_STAT_FIRE = 3wayx3
	SHIP_ACT_RUSH
			SHIP_STAT_FIRE = q
	SHIP_ACT_BACK
			SHIP_STAT_NORMAL = ޑҋ@

	SHIP_ACT_CHARGE
		SHIP_STAT_MOVE   = @ʈړ
		SHIP_STAT_NORMAL = ҋ@
	SHIP_ACT_DISCHARGE
		SHIP_STAT_FIRE   = [U[A~TCAq
		SHIP_STAT_NORMAL = ҋ@

	ȌAJԂB
 */

void boss2_init(SHIP_INFO *info) {

	REPORT(format("info=%p, id=%d", info, info->id));
	system_info.status |= SYSTEM_STAT_BOSS2;
	info->flag |= SHIP_FLAG_NODAMAGE|SHIP_FLAG_HEAVY|SHIP_FLAG_NOCLIP;
	info->count = 150;
	info->sprite.x  = IntToReal(275);
	info->sprite.y  = IntToReal(HEIGHT - 75);
	info->sprite.dx = IntToReal(2)>>1;
	info->sprite.dy = 0;
}

void boss2(SHIP_INFO *info, SHOT_MGR *shot_mgr, SHIP_INFO *ship) {

	switch (info->stat) {
	case SHIP_STAT_APPEAR:
		REPORT("SHIP_STAT_APPEAR");
		info->sprite.y  = IntToReal(HEIGHT - 75);
		if (info->count < 0) {
			info->flag &= ~(SHIP_FLAG_NODAMAGE|SHIP_FLAG_USERS);
			info->stat = SHIP_STAT_MOVE;
			info->count = 60;
			info->sprite.dx = 0;

			/* Nオ */
			info->flag |= SHIP_ACT_MOVE;
			info->sprite.dy = ToReal(-2);
		}
		break;

	case SHIP_STAT_MOVE:
		REPORT("SHIP_STAT_MOVE");
		if (info->count < 0) {
			/* Ԃ玟̃ANV */
			info->stat = SHIP_STAT_NORMAL;
			info->sprite.dy = 0;
			info->count = 60 * 2;
		}

		if (info->flag & SHIP_ACT_MOVE) {
			if (info->sprite.dy < 0 && info->sprite.y < IntToReal(-220)
				|| info->sprite.dy > 0 && info->sprite.y > IntToReal(132)) {
				/* ʒ[Ŕ] */
				info->sprite.dy = -info->sprite.dy;
			}

		} else {

			SURFACE *surface_info = surface_get(info->sprite.surface_id);
			SURFACE *surface_ship = surface_get(ship->sprite.surface_id);

			Real y0 = info->sprite.y + (IntToReal(surface_info->frame_height)>>1);
			Real y1 = ship->sprite.y + (IntToReal(surface_ship->frame_height)>>1);
			Real dy = y1 - y0;

			if (-8 > dy || dy > 8) {
				/* @ɕύX */
				info->sprite.dy = -info->sprite.dy;
			} else {
				/* ̃ANV */
				info->sprite.dy = 0;
				info->count = 60 * 1;
				info->stat  = SHIP_STAT_NORMAL;
			}
		}

		if (!(info->count & ((system_info.level > 10)? 0x07: 0x0f))) {
			/* [U[Vbg̔ */
			SHOT_INFO *shot1 = shot_new(shot_mgr, SHOT_ID_RAY1);
			SHOT_INFO *shot2 = shot_new(shot_mgr, SHOT_ID_RAY1);

			if (shot1 != NULL) {
				shot1->sprite.x = info->sprite.x + IntToReal(245);
				shot1->sprite.y = info->sprite.y + IntToReal(168);
				shot1->sprite.dx = IntToReal(3);
			}

			if (shot2 != NULL) {
				shot2->sprite.x = info->sprite.x + IntToReal(245);
				shot2->sprite.y = info->sprite.y + IntToReal(170);
				shot2->sprite.dx = IntToReal(3);
			}
		}
		break;

	case SHIP_STAT_NORMAL:
		REPORT("SHIP_STAT_NORMAL");
		if (info->count < 0) {
			info->sprite.dy = 0;
			info->count = 5;

			if (info->flag & (SHIP_ACT_BACK|SHIP_ACT_DISCHARGE)) {

				if (info->sprite.x < IntToReal(8)) {

					int rnd = (int)Random(-3, 3);

					info->flag &= ~SHIP_FLAG_USERS;
					info->stat = SHIP_STAT_MOVE;
					info->sprite.dx = 0;

					if (rnd < 0) {
						info->flag |= SHIP_ACT_MOVE;
						info->sprite.dy = -IntToReal(2);
						info->count = 120;

					} else if (rnd > 0) {
						info->flag |= SHIP_ACT_MOVE;
						info->sprite.dy = IntToReal(2);
						info->count = 120;

					} else {
						info->flag |= SHIP_ACT_CHARGE;
					}
				}

			} else if (info->flag & SHIP_ACT_CHARGE) {

				info->stat = SHIP_STAT_FIRE;
				info->flag &= ~SHIP_ACT_MASK;
				info->flag |= SHIP_ACT_DISCHARGE;
				info->sprite.dx = ToReal(-0.5);

			} else {

				info->stat = SHIP_STAT_FIRE;

				/* 3way x 3ƌq̑I */
				if (system_info.level > 7 && Random(0, 4) < 2) {
					info->flag &= ~SHIP_ACT_MASK;
					info->flag |= SHIP_ACT_RUSH;
					//info->sprite.dx = ToReal(0.5);
				}
			}
		}
		break;

	case SHIP_STAT_FIRE:
		REPORT("SHIP_STAT_FIRE");
		if (info->count < 0) {

			info->count = 5;

			if (info->flag & SHIP_ACT_RUSH) {

				if (Random(-2, 18) > system_info.level) {
					/* q̔ */
					SHOT_INFO *shot = shot_new(shot_mgr, SHOT_ID_PHOTON);

					if (shot != NULL) {
						shot_fire(shot, info, 0, 0);
						shot->sprite.dy = IntToReal(-8);
						shot->sprite.dx = IntToReal((int)Random(-8, 8));
					}
				}

				if (info->sprite.dy > 0 && info->sprite.y > IntToReal(HEIGHT - 16)) {
					info->flag &= ~SHIP_ACT_MASK;
					info->flag |= SHIP_ACT_BACK;
					info->stat = SHIP_STAT_NORMAL;
					info->count = 60 * 1;
					info->sprite.dx = -IntToReal(1);
					info->sprite.dy = 0;
				}

			} else if (info->flag & SHIP_ACT_DISCHARGE) {

				//if (info->count & 0x04)
				{
					/* [U[̔ */
					SHOT_INFO *shot = shot_new(shot_mgr, SHOT_ID_RAY2);

					if (shot != NULL) {
						shot->sprite.x = info->sprite.x + IntToReal(176);
						shot->sprite.y = info->sprite.y + IntToReal(58);
						shot->sprite.dx = IntToReal(3);
					}
				}

				if (Random(1, 18) < system_info.level) {
					/* q̔ */
					SHOT_INFO *shot = shot_new(shot_mgr, SHOT_ID_PHOTON);

					if (shot != NULL) {
						shot_fire(shot, info, 0, 0);
						shot->sprite.dy = IntToReal(-8);
						shot->sprite.dx = IntToReal((int)Random(-8, 8));
					}
				}

				if (Random(-2, 18) < system_info.level) {
					/* ~TC̔ */
					SURFACE *surface_info = surface_get(info->sprite.surface_id);
					SURFACE *surface_ship = surface_get(TEX_AXE);

					Real x0 = info->sprite.x + (IntToReal(surface_info->frame_width) >> 1);
					Real y0 = info->sprite.y + (IntToReal(surface_info->frame_height) >> 1);
					Real x1 = IntToReal(surface_ship->frame_width) >> 1;
					Real y1 = IntToReal(surface_ship->frame_height) >> 1;

					enemy_new(&enemy_mgr, SHIP_ID_AXE,
							  x0 - IntToReal(30) - x1,
							  y0 + IntToReal(15) - y1);
					enemy_new(&enemy_mgr, SHIP_ID_AXE,
							  x0 + IntToReal(30) - x1,
							  y0 + IntToReal(15) - y1);
				}

				if (Random(-2, 18) > system_info.level) {
					info->stat = SHIP_STAT_NORMAL;
					info->count = 60 * 2;
					//info->sprite.dy = IntToReal(-1)>>1;
					info->sprite.dy = 0;
				}

			} else if (Random(0, 128) > system_info.level) {
				/* 3way x 3e */

				/* @Zo */
				double angle = sprite_get_angle(&info->sprite, &ship->sprite);
				SHOT_INFO *shot1 = shot_new(shot_mgr, SHOT_ID_BALLET);
				SHOT_INFO *shot2 = shot_new(shot_mgr, SHOT_ID_BALLET);
				SHOT_INFO *shot3 = shot_new(shot_mgr, SHOT_ID_BALLET);

				SHOT_INFO *shot4 = shot_new(shot_mgr, SHOT_ID_BALLET);
				SHOT_INFO *shot5 = shot_new(shot_mgr, SHOT_ID_BALLET);
				SHOT_INFO *shot6 = shot_new(shot_mgr, SHOT_ID_BALLET);

				SHOT_INFO *shot7 = shot_new(shot_mgr, SHOT_ID_BALLET);
				SHOT_INFO *shot8 = shot_new(shot_mgr, SHOT_ID_BALLET);
				SHOT_INFO *shot9 = shot_new(shot_mgr, SHOT_ID_BALLET);

				if (shot1 != NULL) {
					shot_fire(shot1, info, 0, 0);
					sprite_set_angle(&shot1->sprite, angle, IntToReal(2));
				}
				if (shot2 != NULL) {
					shot_fire(shot2, info, 0, 0);
					sprite_set_angle(&shot2->sprite, angle - 15, IntToReal(2));
				}
				if (shot3 != NULL) {
					shot_fire(shot3, info, 0, 0);
					sprite_set_angle(&shot3->sprite, angle + 15, IntToReal(2));
				}

				if (shot4 != NULL) {
					shot_fire(shot4, info, IntToReal(-25), 0);
					sprite_set_angle(&shot4->sprite, angle - 30, IntToReal(2));
				}
				if (shot5 != NULL) {
					shot_fire(shot5, info, IntToReal(-25), 0);
					sprite_set_angle(&shot5->sprite, angle - 45, IntToReal(2));
				}
				if (shot6 != NULL) {
					shot_fire(shot6, info, IntToReal(-25), 0);
					sprite_set_angle(&shot6->sprite, angle - 60, IntToReal(2));
				}

				if (shot7 != NULL) {
					shot_fire(shot7, info, IntToReal(25), 0);
					sprite_set_angle(&shot7->sprite, angle + 30, IntToReal(2));
				}
				if (shot8 != NULL) {
					shot_fire(shot8, info, IntToReal(25), 0);
					sprite_set_angle(&shot8->sprite, angle + 45, IntToReal(2));
				}
				if (shot9 != NULL) {
					shot_fire(shot9, info, IntToReal(25), 0);
					sprite_set_angle(&shot9->sprite, angle + 60, IntToReal(2));
				}

				if (Random(-4, 18) > system_info.level) {
					info->stat = SHIP_STAT_MOVE;
					info->flag |= SHIP_ACT_MOVE;
					info->count = 60;
					info->sprite.dx = 0;

					if (Random(-2, 2) < 0) {
						info->sprite.dy = IntToReal(-2);
					} else {
						info->sprite.dy = IntToReal(2);
					}
				}
			}
		}
		break;

	default:
		REPORT("illegal action!");
		break;
	}
}

void boss2_term(SHIP_INFO *info) {

	REPORT(format("info=%p", info));
	system_info.status &= ~SYSTEM_STAT_BOSS2;
}
