/*
 * effect.c
 *
 * Last modified: <2008/08/27 03:46:09 +0900> By SUMIDA
 */
#include <stdlib.h>
#include <SDL.h>
#include "system.h"
#include "texture.h"
#include "effect.h"

typedef struct EFFECT_TYPE {
	int     tex_id;
	Uint32 options;
	int frame_start;
	int frame_end;
	int frame_interval;
} EFFECT_TYPE;

EFFECT_TYPE effect_type[] = {
	{TEX_EXPLOSION, SPRITE_FRAME64|SPRITE_USE_ANIME, 0, 14, 1},
	{TEX_BOMB     , SPRITE_FRAME|SPRITE_USE_ANIME  , 0, 15, 1},
	{TEX_DISPERSE , SPRITE_FRAME|SPRITE_USE_ANIME  , 0, 15, 1},
	{TEX_POWERUP  , SPRITE_FRAME|SPRITE_USE_ANIME  , 0, 15, 1},
	{TEX_HIT      , SPRITE_FRAME32|SPRITE_USE_ANIME|SPRITE_USE_MOVE, 0, 17, 0},
	{TEX_HIT2     , SPRITE_FRAME32|SPRITE_USE_ANIME|SPRITE_USE_MOVE, 0, 17, 0},
	{TEX_STARS1   , SPRITE_FRAME64|SPRITE_USE_ANIME|SPRITE_USE_MOVE, 0, 2, 1},
	{TEX_STARS2   , SPRITE_FRAME64|SPRITE_USE_ANIME|SPRITE_USE_MOVE|10, 0, 2, 30},
};

EFFECT_MGR effect_mgr;
EFFECT_INFO effect[EFFECT_MAX];

void effect_init(void) {

	size_t number = NUMBEROF(effect);
	size_t i;

	memset(&effect_mgr, 0, sizeof(effect_mgr));
	memset(effect, 0, sizeof(effect));

	effect_mgr.pool = effect;

	for (i = 0; i < number - 1; i++) {
		effect[i].next = &effect[i + 1];
	}
}

EFFECT_INFO *effect_new(EFFECT_ID id) {

	EFFECT_INFO *info = effect_mgr.pool;

	if (info != NULL) {
		EFFECT_TYPE *p = &effect_type[id];

		effect_mgr.pool   = info->next;
		info->next        = effect_mgr.active;
		effect_mgr.active = info;

		info->id                    = id;
		info->sprite.surface_id     = p->tex_id;
		info->sprite.options        = SPRITE_VISIBLE|p->options;
		info->sprite.frame_start    = p->frame_start;
		info->sprite.frame_end      = p->frame_end;
		info->sprite.frame_interval = p->frame_interval;
	}

	return info;
}

void effect_delete(EFFECT_INFO *info, EFFECT_INFO *prev) {

	if (prev == NULL) {
		effect_mgr.active = info->next;
	} else {
		prev->next = info->next;
	}

	memset(info, 0, sizeof(EFFECT_INFO));
	info->next = effect_mgr.pool;
	effect_mgr.pool = info;
}

void effect_action(void) {

	EFFECT_INFO *prev = NULL;
	EFFECT_INFO *info = effect_mgr.active;

	while (info != NULL) {

		if (info->sprite.options & SPRITE_STOP_ANIME) {
			EFFECT_INFO *next = info->next;
			effect_delete(info, prev);
			info = next;
			continue;
		}

		{
			SURFACE *surface = surface_get(info->sprite.surface_id);

			int x0 = RealToInt(info->sprite.x);
			int y0 = RealToInt(info->sprite.y);
			int x1 = x0 + surface->frame_width  -1;
			int y1 = y0 + surface->frame_height -1;

			if (x0 > CLIP_X1 || x1 < CLIP_X0 || y0 > CLIP_Y1 || y1 < CLIP_Y0) {
				EFFECT_INFO *next = info->next;
				effect_delete(info, prev);
				info = next;
				continue;
			}
		}

		sprite_action(&info->sprite);
		sprite_put(&info->sprite);

		prev = info;
		info = info->next;
	}
}

void effect_clean(void) {

	effect_init();
}

void effect_set_pos(EFFECT_INFO *info, SPRITE *spr, Real ox, Real oy) {

	SURFACE *surface_effect = surface_get(info->sprite.surface_id);
	SURFACE *surface_spr = surface_get(spr->surface_id);

	info->sprite.x = spr->x + ox
		+ (IntToReal(surface_spr->frame_width - surface_effect->frame_width) >> 1);
	info->sprite.y = spr->y + oy
		+ (IntToReal(surface_spr->frame_height - surface_effect->frame_height) >> 1);
}
