/*
 * item.c
 *
 * Last modified: <2008/08/31 07:23:45 +0900> By SUMIDA
 */
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <SDL.h>
#include "system.h"
#include "item.h"
#include "myship.h"
#include "enemy.h"
#include "sprite.h"
#include "effect.h"
#include "radian.h"
#include "sound.h"

#define STAR_BASE      45
#define STAR_INTERVAL  10
#define STAR_SPEED     2

extern MYSHIP_MGR myship_mgr;
extern SHIP_MGR enemy_mgr;
extern SHOT_MGR enemy_shot_mgr;
extern SHOT_INFO enemy_shot[ENEMY_SHOT_MAX];

static void masic_change_star(ITEM_INFO *item);

void item_action(ITEM_INFO *item) {

	SHIP_INFO *ship = myship_mgr.ship;

	if (ship->stat != SHIP_STAT_EXPLOSION && ship->stat != SHIP_STAT_LOST) {

		EFFECT_INFO *effect;

		REPORT(format("get item! (item=%p, id=%d, point=%d)", item, item->id, item->point));
		system_info.score += item->point;

		switch (item->id) {
		case ITEM_ID_POWERUP:
			ship->flag |= ITEM_FLAG_POWERUP;
			myship_mgr.ability.powerup = TIME_POWERUP;
			effect = effect_new(EFFECT_ID_POWERUP);
			effect_set_pos(effect, &ship->sprite, 0, 0);
			/*sound_effect(SE_BOMB0);*/
			break;

		case ITEM_ID_SPEEDUP:
			ship->flag |= ITEM_FLAG_SPEEDUP;
			myship_mgr.ability.speedup = TIME_SPEEDUP;
			effect = effect_new(EFFECT_ID_POWERUP);
			effect_set_pos(effect, &ship->sprite, 0, 0);
			break;

		case ITEM_ID_MISSILE:
			ship->flag |= ITEM_FLAG_MISSILE;
			myship_mgr.ability.missile = TIME_MISSILE;
			effect = effect_new(EFFECT_ID_POWERUP);
			effect_set_pos(effect, &ship->sprite, 0, 0);
			break;

		case ITEM_ID_3WAY:
			ship->flag |= ITEM_FLAG_3WAY;
			myship_mgr.ability.threeway = TIME_3WAY;
			effect = effect_new(EFFECT_ID_POWERUP);
			effect_set_pos(effect, &ship->sprite, 0, 0);
			break;

		case ITEM_ID_CHANGE:
			masic_change_star(item);
		case BONUS_ID_STAR:
		case BONUS_ID_SSTAR:
			effect = effect_new(EFFECT_ID_HIT);
			effect->sprite.dx = ToReal(Random(-2.5, 4));
			effect->sprite.dy = ToReal(Random(-2.5, 2.5));
			effect_set_pos(effect, &ship->sprite, 0, 0);
			break;

		default:
			REPORT(format("error item(%d)", item->id));
			break;
		}
	}
}

void masic_change_star(ITEM_INFO *item) {

	SHIP_INFO *enemy;
	SHOT_INFO *shot;

	for (enemy = enemy_mgr.active; enemy != NULL; enemy = enemy->next) {

		if (enemy != item
			&& enemy->id < ITEM_ID_CHANGE
			&& enemy->stat != SHIP_STAT_LOST
			&& enemy->stat != SHIP_STAT_EXPLOSION) {

			int strength = item->strength;

			REPORT(format("item->strength=%d, enemy->strength=%d", item->strength, enemy->strength));

			//system_info.score += ship_down(enemy, &item->strength, 0, 0);
			system_info.score += ship_down(enemy, &strength, 0, 0);

			if (enemy->strength <= 0) {
				int count = (enemy->id == SHIP_ID_BOSS1
							 || enemy->id == SHIP_ID_BOSS2) ? 10: 1;

				while (--count >= 0) {
					SHIP_INFO *item = enemy_new(&enemy_mgr, BONUS_ID_STAR, 0, 0);
					EFFECT_INFO *effect = effect_new(EFFECT_ID_HIT);

					if (item != NULL) {
						double angle = DegToRad(STAR_BASE + count * STAR_INTERVAL * ((count & 1)? -1: 1));
						item->sprite.dx = ToReal(sin(angle) * STAR_SPEED);
						item->sprite.dy = ToReal(-cos(angle) * STAR_SPEED);
						sprite_set_pos(&item->sprite, &enemy->sprite, 0, 0);
						REPORT(format("item x=%d, y=%d", RealToInt(item->sprite.x), RealToInt(item->sprite.y)));
					} else {
						REPORT("item create fail!");
					}

					if (effect != NULL) {
						effect_set_pos(effect, &enemy->sprite, 0, 0);
						effect->sprite.dx = ToReal(Random(-2.5, 4));
						effect->sprite.dy = ToReal(Random(-2.5, 2.5));
					}
				}
			}
		}
	}

	for (shot = enemy_shot_mgr.active; shot != NULL; shot = shot->next) {

		SHIP_INFO *item = enemy_new(&enemy_mgr, BONUS_ID_SSTAR, 0, 0);
		EFFECT_INFO *effect = effect_new(EFFECT_ID_HIT);

		if (item != NULL) {
			item->sprite.dx = ToReal(1);
			item->sprite.dy = ToReal(-1);
			sprite_set_pos(&item->sprite, &shot->sprite, 0, 0);
			//REPORT(format("item x=%d, y=%d", RealToInt(item->sprite.x), RealToInt(item->sprite.y)));
		} else {
			REPORT("item create fail!");
		}

		if (effect != NULL) {
			effect_set_pos(effect, &shot->sprite, 0, 0);
			effect->sprite.dx = ToReal(Random(-2.5, 4));
			effect->sprite.dy = ToReal(Random(-2.5, 2.5));
		}
	}

	shot_init(&enemy_shot_mgr, enemy_shot, NUMBEROF(enemy_shot));
}

void item_init(SHIP_INFO *info) {

	REPORT(format("info=%p, id=%d", info, info->id));
	info->flag |= SHIP_FLAG_ITEM;
	info->sprite.dx = IntToReal(2);
	info->sprite.dy = IntToReal(-4);
	info->sprite.frame_start    = 0;
	info->sprite.frame_end      = 2;
	info->sprite.frame_interval = 10;
}

void item(SHIP_INFO *info, SHOT_MGR *shot_mgr, SHIP_INFO *ship) {

	if (info->id < BONUS_ID_STAR) {
		info->sprite.dy += ACCELERATION;
	} else {
		info->sprite.dy += ACCELERATION >> 2;
	}
}

void item_term(SHIP_INFO *info) {
	REPORT(format("info=%p", info));
}
