/*
 * shot.c
 *
 * Last modified: <2008/08/27 03:45:09 +0900> By SUMIDA
 */
#include <stdlib.h>
#include <math.h>
#include <SDL.h>
#include "system.h"
#include "texture.h"
#include "surface.h"
#include "sprite.h"
#include "shot.h"
#include "effect.h"

typedef struct SHOT_TYPE {
	int tex_id;
	int strength;
	Uint32 options;
	int frame_start;
	int frame_end;
	int frame_interval;
} SHOT_TYPE;

SHOT_TYPE shot_type[] = {
	{TEX_SHOT     ,  1,   0|SPRITE_FRAME32|SPRITE_USE_ANIME|SPRITE_LOOP_ANIME, 2, 3, 0},
	{TEX_SHOT2    ,  2,   0|SPRITE_FRAME32|SPRITE_USE_ANIME|SPRITE_LOOP_ANIME, 2, 3, 0},
	{TEX_MISSILE  ,  5,   0|SPRITE_FRAME|SPRITE_USE_MOVE, 0, 0, 0},
	{TEX_BALLET   ,  1,   0|SPRITE_FRAME8|SPRITE_USE_ANIME|SPRITE_LOOP_ANIME, 0, 3, 2},
	{TEX_RAY1     ,  5, 100|SPRITE_FRAME, 0, 0, 0},
	{TEX_RAY2     ,  5, 100|SPRITE_FRAME, 0, 0, 0},
	{TEX_PHOTON   , 10,   0|SPRITE_FRAME|SPRITE_USE_ANIME|SPRITE_LOOP_ANIME, 0, 3, 8},
};

static void missile(SHOT_INFO* info, SHIP_INFO* ship);
static void photon(SHOT_INFO* info, SHIP_INFO* ship);

void(*func_shot[])(SHOT_INFO*, SHIP_INFO*) = {
	NULL,
	NULL,
	missile,
	NULL,
	NULL,
	NULL,
	photon,
};

void shot_init(SHOT_MGR *mgr, SHOT_INFO *infos, size_t number) {

	size_t i;

	if (mgr != NULL) {
		memset(mgr, 0, sizeof(SHOT_MGR));
		mgr->pool = infos;
	}

	memset(infos, 0, sizeof(SHOT_INFO) * number);

	for (i = 0; i < number - 1; i++) {
		infos[i].next = &infos[i + 1];
	}
}

SHOT_INFO *shot_new(SHOT_MGR *mgr, SHOT_ID id) {

	SHOT_INFO *info = mgr->pool;

	if (info != NULL) {
		SHOT_TYPE *p = &shot_type[id];
		SURFACE *surface = surface_get(p->tex_id);

		mgr->pool   = info->next;
		info->next  = mgr->active;
		mgr->active = info;

		info->id       = id;
		info->stat     = SHOT_STAT_NORMAL;
		info->count    = 0;
		info->strength = p->strength;
		info->sprite.surface_id = p->tex_id;
		info->sprite.options = SPRITE_VISIBLE|SPRITE_USE_MOVE|SPRITE_USE_COLLISION|p->options;
		info->sprite.frame_start    = p->frame_start;
		info->sprite.frame_end      = p->frame_end;
		info->sprite.frame_interval = p->frame_interval;
		info->sprite.collision.x0 = IntToReal(0);
		info->sprite.collision.y0 = IntToReal(0);
		info->sprite.collision.x1 = IntToReal(surface->frame_width - 1);
		info->sprite.collision.y1 = IntToReal(surface->frame_height - 1);
	}

	return info;
}

void shot_delete(SHOT_MGR *mgr, SHOT_INFO *info, SHOT_INFO *prev) {

	if (prev == NULL) {
		mgr->active = info->next;
	} else {
		prev->next = info->next;
	}

	memset(info, 0, sizeof(SHOT_INFO));
	info->next = mgr->pool;
	mgr->pool = info;
}

int shot_action(SHOT_MGR *mgr, SHIP_MGR *ship_mgr) {

	SHOT_INFO *prev = NULL;
	SHOT_INFO *info = mgr->active;
	SHIP_INFO *ship;

	int score = 0;

	while (info != NULL) {

		SPRITE *sprite = &info->sprite;
#if 0
		/* ~߂Ăꍇ͍폜 */
		if (!(sprite->options & SPRITE_USE_MOVE)) {
			SHOT_INFO *next = info->next;
			shot_delete(mgr, info, prev);
			info = next;
			continue;
		}
#endif

		/* Vbg̓ꏈ */
		if (func_shot[info->id] != NULL) {
			func_shot[info->id](info, ship_mgr->active);
		}

		/* Vbg̈ړ */
		sprite_action(&info->sprite);

		/* ͂ݏoꍇ͍폜 */
		{
			SURFACE *surface = surface_get(info->sprite.surface_id);
			int x0 = RealToInt(info->sprite.x);
			int y0 = RealToInt(info->sprite.y);
			int x1 = x0 + surface->frame_width  -1;
			int y1 = y0 + surface->frame_height -1;

			if (x0 > CLIP_X1 || x1 < CLIP_X0 || y0 > CLIP_Y1 || y1 < CLIP_Y0) {
				SHOT_INFO *next = info->next;
				shot_delete(mgr, info, prev);
				info = next;
				continue;
			}
		}

		/* G@Ƃ̐ڐG */
		for (ship = ship_mgr->active; ship != NULL; ship = ship->next) {

			if (!(ship->flag & SHIP_FLAG_ITEM)
				&& ship->stat != SHIP_STAT_LOST
				&& ship->stat != SHIP_STAT_EXPLOSION
				&& sprite_collision_check(&info->sprite, &ship->sprite) == SDL_TRUE) {

				/*REPORT("hit!\n");*/
				score += ship_down(ship, &info->strength, info->sprite.dx >> 3, info->sprite.dy >> 3);

				if (info->strength <= 0) {
					if (info->id == SHOT_ID_NORMAL
						|| info->id == SHOT_ID_MISSILE) {

						EFFECT_INFO *effect = effect_new(EFFECT_ID_HIT);

						if (effect != NULL) {
							effect_set_pos(effect, sprite, 0, 0);
							effect->sprite.dx = ToReal(Random(-2.5, 4));
							effect->sprite.dy = ToReal(Random(-2.5, 2.5));
						}
					}

					else if (info->id == SHOT_ID_NORMAL2) {

						EFFECT_INFO *effect = effect_new(EFFECT_ID_HIT2);

						if (effect != NULL) {
							effect_set_pos(effect, sprite, 0, 0);
							effect->sprite.dx = ToReal(Random(-2.5, 4));
							effect->sprite.dy = ToReal(Random(-2.5, 2.5));
						}
					}

					{
						SHOT_INFO *next = info->next;
						shot_delete(mgr, info, prev);
						info = next;
						sprite = NULL;
						/*REPORT("shot delete!\n");*/
					}
					break;
				}
			}
		}

		/* ڐG̏ꍇ͕\ */
		if (sprite != NULL) {
			sprite_put(sprite);
			prev = info;
			info = info->next;
		}
	}

	return score;
}

void shot_fire(SHOT_INFO *info, SHIP_INFO *ship, Real ox, Real oy) {

	SURFACE *surface_shot = surface_get(info->sprite.surface_id);
	SURFACE *surface_ship = surface_get(ship->sprite.surface_id);

	info->sprite.x = ship->sprite.x + ox
		+ (IntToReal(surface_ship->frame_width - surface_shot->frame_width) >> 1);
	info->sprite.y = ship->sprite.y + oy
		+ (IntToReal(surface_ship->frame_height - surface_shot->frame_height) >> 1);
}

void shot_homing(SPRITE *info, SPRITE *ship, Real speed) {

	SURFACE *surface_ship = surface_get(ship->surface_id);
	SURFACE *surface_shot = surface_get(info->surface_id);
	double dx = (ship->x + (IntToReal(surface_ship->frame_width) >> 1)
				 - info->x - (IntToReal(surface_shot->frame_width) >> 1)) / 65536.0;
	double dy = (ship->y + (IntToReal(surface_ship->frame_height) >> 1)
				 - info->y - (IntToReal(surface_shot->frame_height) >> 1)) / 65536.0;
	double d = sqrt(dx * dx + dy * dy);

	if (d) {
		double spd = speed / 65536.0;
		info->dx = (Real)((dx / d * spd) * 65536);
		info->dy = (Real)((dy / d * spd) * 65536);
	} else {
		info->dx = 0;
		info->dy = speed;
	}
}

/* }WbNE~TC */
static void missile(SHOT_INFO* info, SHIP_INFO* ship) {

	if (info->sprite.dx > IntToReal(-24)) {
		info->sprite.dx -= ToReal(0.5);
	}
	info->sprite.dy -= (info->sprite.dy >> 4);
}

/* z[~Oq */
static void photon(SHOT_INFO* info, SHIP_INFO* ship) {

	if (info->count++ < 60 * 3) {

		Real dx = info->sprite.dx;
		Real dy = info->sprite.dy;

		shot_homing(&info->sprite, &ship->sprite, IntToReal(1));

		info->sprite.dx = (info->sprite.dx + dx * 7) >> 3;
		info->sprite.dy = (info->sprite.dy + dy * 7) >> 3;

	} else if(!(info->count & 0x03)) {
		if (RealMul(info->sprite.dx, info->sprite.dx)
			   + RealMul(info->sprite.dy, info->sprite.dy) < IntToReal(16)) {
			info->sprite.dx += info->sprite.dx >> 4;
			info->sprite.dy += info->sprite.dy >> 4;
		}
	}
}
