/*
 * sound.c
 *
 * Last modified: <2008/08/07 02:52:46 +0900> By SUMIDA
 */
#include <stdlib.h>
#include <SDL.h>
#include <SDL_mixer.h>
#include "system.h"
#include "sound.h"

/*#define USE_AUDIO /**/

#ifdef USE_AUDIO
static char music_name[BGM_MAX][_MAX_PATH];
static Mix_Music* music;
static int channel_no;
static Mix_Chunk* chunk[CHUNK_MAX];
#else
static int dummy_play_count = 0;
#endif

/*  */
void sound_init(void) {
#ifdef USE_AUDIO
	printf("sound_init ... ");
	music = NULL;
	channel_no = 0;
	memset(chunk, NULL, sizeof(chunk));

	Mix_AllocateChannels(CHANNEL_MAX);
	printf("done.\n");
#endif
}

void sound_load_music(const char* file, BGM_ID id) {
#ifdef USE_AUDIO
	printf("music_load(%s) ... ", file);
	snprintf(music_name[id], _MAX_PATH, "%s", file);
	printf("done.\n");
#endif
}

void sound_load_effect(const char* file, SE_ID id) {
#ifdef USE_AUDIO
	printf("effect_load(%s) ... ", file);
	chunk[id] = Mix_LoadWAV(file);
	printf("done.\n");
#endif
}

/* I */
void sound_free(void) {
#ifdef USE_AUDIO
	int i;

	printf("sound_free ... ");

	/* t~ */
	sound_stop_music();

	/* ʉ̉ */
	for (i = 0; i < NUMBEROF(chunk); i++) {
		if (chunk[i] != NULL) {
			Mix_FreeChunk(chunk[i]);
		}
	}

	Mix_CloseAudio();

	printf("done.\n");
#endif
}

void sound_music(BGM_ID id, int loop) {
#ifdef USE_AUDIO
	//printf("sound_music(%s, %d) ... ", id, loop);
	sound_stop_music();
	music = Mix_LoadMUS(music_name[id]);
	Mix_PlayMusic(music, loop);
	//printf("done.\n");
#else
	dummy_play_count = 0;
#endif
}

void sound_stop_music(void) {
#ifdef USE_AUDIO
	//printf("sound_stop_music ... ");
	if (music != NULL) {
		Mix_HaltMusic();
		Mix_FreeMusic(music);
		music = NULL;
	}
	//printf("done.\n");
#endif
}

void sound_effect(SE_ID id) {
#ifdef USE_AUDIO
	//printf("sound_effect(%d) ... ", id);
	Mix_PlayChannel(-1, chunk[id], 0);
	//printf("done.\n");
#endif
}

void sound_stop_effect(void) {
#ifdef USE_AUDIO
	//printf("sound_stop_effect ... ");
	Mix_HaltChannel(-1);
	channel_no = 0;
	//printf("done.\n");
#endif
}

int sound_playing_music(void) {
#ifdef USE_AUDIO
	return (music != NULL && Mix_PlayingMusic());
#else
	REPORT(format("dummy_play_count=%d", dummy_play_count));
	return (++dummy_play_count < 10 * 60);
#endif
}
